<?php
/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2017 All rights reserved
 * @package DZF
 */

#namespace DeubertIt;

/**
 * @author Eugene Melnik <e.melnik@luckywarepro.com>
 */
class DZF_Settings
{
    const CACHE_KEY = 'dzf_settings_cache';

    /**
     * @var DZF_Settings_Source|array
     */
    protected $_source;

    /**
     * @var \Zend_Cache_Backend
     */
    protected $_cache;

    /**
     * @var int
     */
    protected $_cacheLifetime = 86400; // seconds = 1 day

    /**
     * @var array
     */
    protected static $_configFiles = array(
        APPLICATION_PATH . '/configs/application.ini',
        APPLICATION_PATH . '/configs/local.ini'
    );

    /**
     * @var DZF_Settings
     */
    protected static $_instance;

    /**
     * @return DZF_Settings
     * @throws DZF_Exception
     */
    public static function getInstance()
    {
        if (!isset(self::$_instance))
        {
            $instance = new self();

            $instance->setSource([
                new DZF_Settings_Source_Ini(self::$_configFiles, APPLICATION_ENV),
                new DZF_Settings_Source_Doctrine()
            ]);

            self::$_instance = $instance;
        }

        return self::$_instance;
    }

    /**
     * @param   string          $groupKey
     * @param   string          $paramName      (optional, default - null)
     * @return  mixed
     */
    public static function get($groupKey, $paramName = null)
    {
        $instance = self::getInstance();
        $cachedValue = $instance->getCachedValue($groupKey, $paramName);

        if (is_null($cachedValue))
        {
            $value = $instance->getValue($groupKey, $paramName);
            $instance->setCachedValue($groupKey, $paramName, $value);
            $cachedValue = $value;
        }

        return $cachedValue;
    }

    /**
     * @param   string          $groupKey
     * @param   string          $paramName      (optional, default - null)
     * @return  mixed
     */
    public function getValue($groupKey, $paramName = null)
    {
        if (is_array($this->_source))
        {
            $value = null;

            foreach ($this->_source as $source)
            {
                $tmpValue = $source->getValue($groupKey, $paramName);

                if (!is_null($tmpValue))
                {
                    $value = $tmpValue;
                }
            }
        }
        else
        {
            $value = $this->_source->getValue($groupKey, $paramName);
        }

        return $value;
    }

    /**
     * @param   string          $groupKey
     * @param   string          $paramName      (optional, default - null)
     * @return  mixed
     */
    public function getCachedValue($groupKey, $paramName = null)
    {
        if (!isset($this->_cache) || $this->_cache->test(self::CACHE_KEY) === false)
        {
            return null;
        }

        try
        {
            $cacheData = \Zend_Json::decode($this->_cache->load(self::CACHE_KEY));
        }
        catch (\Exception $e)
        {
            $this->clearCache();
            return null;
        }

        if (is_array($cacheData) && array_key_exists($groupKey, $cacheData))
        {
            if (is_null($paramName))
            {
                return $cacheData[$groupKey];
            }

            if (is_array($cacheData[$groupKey]) && array_key_exists($paramName, $cacheData[$groupKey]))
            {
                return $cacheData[$groupKey][$paramName];
            }
        }

        return null;
    }

    /**
     * @param   string          $groupKey
     * @param   string          $paramName
     * @param   mixed           $value
     * @return  DZF_Settings
     */
    public function setCachedValue($groupKey, $paramName, $value)
    {
        if (!isset($this->_cache))
        {
            return;
        }

        $cacheData = $this->_cache->load(self::CACHE_KEY);

        if (empty($cacheData))
        {
            $cacheData = array();
        }
        else
        {
            try
            {
                $cacheData = \Zend_Json::decode($cacheData);
            }
            catch (\Exception $e)
            {
                $cacheData = array();
            }
        }

        if (!isset($cacheData[$groupKey]))
        {
            $cacheData[$groupKey] = array();
        }

        if (!is_null($paramName))
        {
            $cacheData[$groupKey][$paramName] = $value;
        }
        else
        {
            $cacheData[$groupKey] = $value;
        }

        $this->_cache->save(\Zend_Json::encode($cacheData), self::CACHE_KEY, array(), $this->_cacheLifetime);

        return $this;
    }

    /**
     * @param   DZF_Settings_Source|array       $source
     * @return  DZF_Settings
     * @throws  DZF_Exception
     */
    public function setSource($source)
    {
        if ($source instanceof DZF_Settings_Source || is_array($source))
        {
            $this->_source = $source;
            return $this;
        }
        else
        {
            throw new DZF_Exception('Should be instance of DZF_Settings_Source of array of instances');
        }
    }

    /**
     * @param   \Zend_Cache_Backend      $cache
     * @return  DZF_Settings
     */
    public function setCache($cache)
    {
        $this->_cache = $cache;
        return $this;
    }

    /**
     * @return  \Zend_Cache_Backend|null
     */
    public function getCache()
    {
        return $this->_cache;
    }

    /**
     * @return  DZF_Settings
     */
    public function clearCache()
    {
        if (isset($this->_cache))
        {
            $this->_cache->remove(self::CACHE_KEY);
        }

        return $this;
    }
}
